﻿//-----------------------------------------------------------------------------
// <copyright file="Program.cs" company="eQ-3 Entwicklung GmbH">
//  Copyright (c) 2013 eQ-3 Entwicklung GmbH
// </copyright>
// <summary>Main-menu of the console-application.</summary>
//-----------------------------------------------------------------------------
namespace DDS130Demo
{
    using System;
    using System.Collections.Generic;
    using System.IO;
    using System.IO.Ports;
    using System.Threading;
    using DDSCommunication;

    /// <summary>
    /// Main-menu of the console-application.
    /// </summary>
    public class Program
    {
        /// <summary>
        /// Indicates whether a DDS30/DDS130 is currently connected or not.
        /// </summary>
        private static bool ddsConnectionEnabled;

        /// <summary>
        /// Current DDS30/DDS130 device.
        /// </summary>
        private static DDS device;

        /// <summary>
        /// Main-method of the console-application.
        /// </summary>
        public static void Main()
        {
            try
            {
                Program.PickDevice();
                Program.ShowMenu();
            }
            catch (Exception)
            {
                Console.Clear();

                Console.WriteLine("FEHLER: Die Verbindung mit dem DDS konnte nicht hergestellt werden.");
                Console.ReadKey(true);
            }
        }

        /// <summary>
        /// Shows the dialog for picking whether DDS30 or DDS130. 
        /// </summary>
        private static void PickDevice()
        {
            Program.ddsConnectionEnabled = false;

            Console.Clear();
            Console.WriteLine("=== Demosoftware für DDS30 und DDS130 ===");
            Console.WriteLine("[1] - DDS30 (VCP)");
            Console.WriteLine("[2] - DDS130 (USB)");
            Console.WriteLine("[9] - Programmende");
            int choice = Program.GetValidChoice(new int[] { 1, 2, 9 });

            switch (choice)
            {
                case 1:
                    Console.WriteLine();
                    var comPort = Program.GetValidComPort();

                    Program.device = new DDS30(comPort);
                    break;

                case 2:
                    Program.device = new DDS130();
                    break;

                default:
                    return;
            }

            Program.device.SetDeviceConnectionState(false);
        }

        /// <summary>
        /// Gets a valid choice.
        /// </summary>
        /// <param name="validInputs">List of valid choices.</param>
        /// <returns>Entered valid input.</returns>
        private static int GetValidChoice(int[] validInputs)
        {
            List<int> validInputsList = new List<int>(validInputs);

            bool isValidInput;
            int input = -1;

            Console.Write("Auswahl: ");
            do
            {
                isValidInput = true;

                try
                {
                    input = int.Parse(Console.ReadLine());
                    if (!validInputsList.Contains(input))
                    {
                        throw new Exception();
                    }
                }
                catch
                {
                    Console.Write("ungültige Eingabe - Auswahl: ");
                    isValidInput = false;
                }
            }
            while (!isValidInput);

            return input;
        }

        /// <summary>
        /// Gets a currently available valid ComPort.
        /// </summary>
        /// <returns>Currently available valid ComPort.</returns>
        private static string GetValidComPort()
        {
            var availableComPorts = SerialPort.GetPortNames();
            int[] availableComPortNumbers = new int[availableComPorts.Length];

            Console.WriteLine("Bitte wählen Sie einen COM für die Verbindung.");
            for (int i = 0; i < availableComPorts.Length; i++)
            {
                var currentAvailableComPort = availableComPorts[i];
                availableComPortNumbers[i] = int.Parse(currentAvailableComPort.Substring(3, currentAvailableComPort.Length - 3));

                Console.WriteLine(string.Format(
                    "[{0}] - {1}",
                    availableComPortNumbers[i],
                    currentAvailableComPort));
            }

            return string.Format("COM{0}", Program.GetValidChoice(availableComPortNumbers));
        }

        /// <summary>
        /// Shows the main menu.
        /// </summary>
        private static void ShowMenu()
        {
            Console.Clear();

            bool run = true;
            do
            {
                Console.WriteLine("=== Demosoftware für DDS30 und DDS130 ===");
                Console.WriteLine();
                Console.WriteLine("[1] - Beispielhafte Geräteverbindung zum DDS30/DDS130 herstellen bzw. unterbrechen");
                Console.WriteLine("[2] - Frequenz ändern");
                Console.WriteLine("[3] - Signalform ändern");
                Console.WriteLine("[4] - Aktuelle Einstellungen ausgeben");
                Console.WriteLine("[9] - Programmende");
                Console.WriteLine();

                int choice = Program.GetValidChoice(new int[] { 1, 2, 3, 4, 9 });
                Console.Clear();

                switch (choice)
                {
                    // connect or disconnect to DDS130
                    case 1:
                        Program.DemonstrateDeviceConnection();
                        break;

                    // change frequency
                    case 2:
                        Program.EditFrequency();
                        break;

                    // change signal form
                    case 3:
                        Program.EditSignalForm();
                        break;

                    case 4:
                        Program.ShowPreferences();
                        break;

                    // exit application
                    default:
                        run = false;
                        break;
                }

                Console.Clear();
            }
            while (run);
        }

        /// <summary>
        /// Displays a connection error and waits 3 seconds.
        /// </summary>
        private static void DisplayConnectionError()
        {
            Console.WriteLine("Fehler: Beim Herstellen bzw. Verbinden zum DDS130 ist ein Fehler aufgetreten.\n");
            Thread.Sleep(3000);
            Console.Clear();
        }

        /// <summary>
        /// Option 1: Connect or disconnect to DDS30/DDS130.
        /// </summary>
        private static void DemonstrateDeviceConnection()
        {
            bool run = true;

            do
            {
                string state = Program.GetDeviceConnectionState();

                Console.WriteLine("=== Geräteverbindung zum DDS30 bzw. DDS130 herstellen bzw. unterbrechen ===");
                Console.WriteLine("Dies dient zur Demonstration, andere Ansteuerungen verwalten die Geräteverbindung automatisch.");
                Console.WriteLine();
                Console.WriteLine("Aktueller Status: {0}", state);
                Console.WriteLine("[1] Status umschalten");
                Console.WriteLine("[9] Hauptmenü");

                int choice = Program.GetValidChoice(new int[] { 1, 9 });

                switch (choice)
                {
                    case 1:
                        Program.ToggleDeviceConnection();
                        break;
                    case 9:
                        // in main menu a connection to a DDS may not be established
                        if (Program.ddsConnectionEnabled)
                        {
                            Program.ToggleDeviceConnection();
                        }

                        run = false;
                        break;
                    default:
                        break;
                }

                Console.Clear();
            }
            while (run);
        }

        /// <summary>
        /// Toggles the connection state of the DDS30/DDS130.
        /// </summary>
        private static void ToggleDeviceConnection()
        {
            bool activate;

            if (Program.ddsConnectionEnabled)
            {
                activate = false;
                Program.ddsConnectionEnabled = false;
            }
            else
            {
                activate = true;
                Program.ddsConnectionEnabled = true;
            }

            if (!Program.device.SetDeviceConnectionState(activate))
            {
                Program.DisplayConnectionError();
            }
        }

        /// <summary>
        /// Gets the current device connection state as string.
        /// </summary>
        /// <returns>Current device connection state as string.</returns>
        private static string GetDeviceConnectionState()
        {
            if (Program.ddsConnectionEnabled)
            {
                return "Geräteverbindung hergestellt";
            }
            else
            {
                return "keine Geräteverbindung hergestellt";
            }
        }

        /// <summary>
        /// Option 2: Changes frequency.
        /// </summary>
        private static void EditFrequency()
        {
            if (!(Program.device != null && Program.device.SetDeviceConnectionState(true)))
            {
                Program.DisplayConnectionError();
                return;
            }

            bool run = true;
            do
            {
                Console.WriteLine("=== Frequenz ändern ===");
                Console.WriteLine();
                Console.WriteLine("Aktuelle Frequenz: {0:,0.##} Hz", Program.GetFrequency());
                Console.WriteLine("[1] Frequenz setzen");
                Console.WriteLine("[9] Hauptmenü");
                Console.WriteLine();

                int choice = Program.GetValidChoice(new int[] { 1, 9 });

                switch (choice)
                {
                    case 1:
                        Program.SetFrequency();
                        break;
                    case 9:
                        run = false;
                        break;
                    default:
                        break;
                }

                Console.Clear();
            }
            while (run);

            if (!Program.device.SetDeviceConnectionState(false))
            {
                Program.DisplayConnectionError();
            }
        }

        /// <summary>
        /// Gets the current frequency of the DDS30/DDS130.
        /// </summary>
        /// <returns>Current frequency of the DDS30/DDS130.</returns>
        private static double GetFrequency()
        {
            try
            {
                return Program.device.ReadFrequency();
            }
            catch (InvalidOperationException ex)
            {
                Console.WriteLine(ex.Message);
                return 0.0;
            }
        }

        /// <summary>
        /// Sends a command to change the current frequency of the DDS30/DDS130.
        /// </summary>
        private static void SetFrequency()
        {
            double frequency = Program.GetFrequencyFromUser();

            try
            {
                if (!Program.device.SetFrequency(frequency))
                {
                    Program.DisplayConnectionError();
                }
            }
            catch (ArgumentException ex)
            {
                Console.WriteLine(ex.Message);
                Console.ReadKey();
            }
        }

        /// <summary>
        /// Gets a valid frequency to send to the DDS130 as command.
        /// </summary>
        /// <returns>Valid frequency.</returns>
        private static double GetFrequencyFromUser()
        {
            for (;;)
            {
                Console.Write("Zu setzende Frequenz in Hz: ");

                double frequency;
                if (double.TryParse(Console.ReadLine().Replace(".", ","), out frequency))
                {
                    return frequency;
                }
            }
        }

        /// <summary>
        /// Option 3: Changes signal form.
        /// </summary>
        private static void EditSignalForm()
        {
            if (!(Program.device != null && Program.device.SetDeviceConnectionState(true)))
            {
                Program.DisplayConnectionError();
                return;
            }

            bool run = true;
            do
            {
                string state = Program.GetCurrentSignalFormAsString();

                Console.WriteLine("=== Signalform ändern ===");
                Console.WriteLine();
                Console.WriteLine("Aktueller Status: {0}", state);
                Console.WriteLine("[1] Sinus");
                Console.WriteLine("[2] Dreieck");
                Console.WriteLine("[3] Rechteck (volle Frequenz)");
                Console.WriteLine("[4] Rechteck (halbe Frequenz)");
                Console.WriteLine("[9] Hauptmenü");
                Console.WriteLine();

                int choice = Program.GetValidChoice(new int[] { 1, 2, 3, 4, 9 });

                switch (choice)
                {
                    case 9:
                        run = false;
                        break;
                    default:
                        if (!Program.ChangeSignalForm(choice))
                        {
                            Console.WriteLine("--> Fehler bei Editierung der Signalform.");
                            Thread.Sleep(2000);
                        }

                        break;
                }

                Console.Clear();
            }
            while (run);

            if (!Program.device.SetDeviceConnectionState(false))
            {
                Program.DisplayConnectionError();
            }
        }

        /// <summary>
        /// Shows the current preferences of the DDS30/DDS130.
        /// </summary>
        private static void ShowPreferences()
        {
            if (!(Program.device != null && Program.device.SetDeviceConnectionState(true)))
            {
                Program.DisplayConnectionError();
                return;
            }

            var ddsPreferences = Program.device.ReadPreferences();

            Console.WriteLine("=== Aktuelle Einstellungen bzw. Konfiguration (Auszug) ===");
            Console.WriteLine(string.Format("Version            : {0}", ddsPreferences.Version));
            Console.WriteLine(string.Format("Betriebsart        : {0}", DDSUtils.DeviceModeToString(ddsPreferences.DeviceMode)));
            Console.WriteLine(string.Format("Frequenz           : {0} Hz", ddsPreferences.Frequency));
            Console.WriteLine(string.Format("Minimalfrequenz    : {0} Hz", ddsPreferences.MinimalFrequency));
            Console.WriteLine(string.Format("Maximalfrequenz    : {0} Hz", ddsPreferences.MaximalFrequency));
            Console.WriteLine(string.Format("Schrittweite       : {0} Hz", ddsPreferences.FrequencyStepSize));
            Console.WriteLine(string.Format("Amplitude          : {0} mV", ddsPreferences.Amplitude));
            Console.WriteLine(string.Format("Wobbelstartfrequenz: {0} Hz", ddsPreferences.WobbleStartFrequency));
            Console.WriteLine(string.Format("Wobbelstoppfrequenz: {0} Hz", ddsPreferences.WobbleStopFrequency));
            Console.WriteLine(string.Format("Wobbelfrequenz     : {0} Hz", ddsPreferences.WobbleFrequency));
            Console.WriteLine(string.Format("Signalform         : {0}", DDSUtils.SignalFormToString(ddsPreferences.DeviceSignalForm)));
            Console.WriteLine();
            Console.Write("Druecken Sie eine beliebige Taste um fortzufahren...");
            Console.ReadKey(true);

            if (!Program.device.SetDeviceConnectionState(false))
            {
                Program.DisplayConnectionError();
            }
        }

        /// <summary>
        /// Gets the current displayed signal form of the DDS30/DDS130.
        /// </summary>
        /// <returns>Current displayed signal form of the DDS30/DDS130.</returns>
        private static string GetCurrentSignalFormAsString()
        {
            return DDSUtils.SignalFormToString(Program.device.ReadSignalForm());
        }

        /// <summary>
        /// Sends a command to the DDS30/DDS130 containing the signal form to display.
        /// </summary>
        /// <param name="choice">Choice of menu-option 3.</param>
        /// <returns>Result of the action.</returns>
        private static bool ChangeSignalForm(int choice)
        {
            return Program.device.SetSignalForm((DDSPreferences.SignalForm)(choice - 1));
        }
    }
}