﻿//-----------------------------------------------------------------------------
// <copyright file="CRC16.cs" company="eQ-3 Entwicklung GmbH">
//  Copyright (c) 2013 eQ-3 Entwicklung GmbH
// </copyright>
// <summary>Class for calculating CRC16-checksums</summary>
//-----------------------------------------------------------------------------
namespace DDSCommunication
{
    using System;

    /// <summary>
    /// Calculates CRC16-checksums.
    /// </summary>
    public class CRC16
    {
        /// <summary>
        /// Polynomial of the CRC16.
        /// </summary>
        public const ushort POLYNOM = 0x8005;

        /// <summary>
        /// Initial value of the checksum.
        /// </summary>
        public const ushort INIT = 0xffff;

        /// <summary>
        /// Current checksum of the CRC16.
        /// </summary>
        private ushort checksum;

        /// <summary>
        /// Initializes a new instance of the CRC16 class.
        /// </summary>
        public CRC16()
        {
            this.Reset();
        }

        /// <summary>
        /// Gets the Current checksum of the CRC16.
        /// </summary>
        /// <value>Current checksum of the CRC16.</value>
        public ushort Checksum
        {
            get { return this.checksum; }
        }

        /// <summary>
        /// Resets the current checksum of the CRC16.
        /// </summary>
        /// <returns>Current checksum as CRC16.</returns>
        public CRC16 Reset()
        {
            this.checksum = INIT;
            return this;
        }

        /// <summary>
        /// Shifts a single byte.
        /// </summary>
        /// <param name="b">Byte to shift.</param>
        /// <returns>Current checksum as CRC16.</returns>
        public CRC16 Process(byte b)
        {
            bool flag;

            for (int i = 0; i < 8; i++)
            {
                flag = (this.checksum & 0x8000) != 0;
                this.checksum <<= 1;
                if ((b & 0x80) == 0x80)
                {
                    this.checksum |= 1;
                }

                b <<= 1;
                if (flag)
                {
                    this.checksum ^= POLYNOM;
                }
            }

            return this;
        }

        /// <summary>
        /// Shifts a byte array.
        /// </summary>
        /// <param name="bytes">Byte-array to shift.</param>
        /// <returns>Current checksum as CRC16.</returns>
        public CRC16 Process(byte[] bytes)
        {
            foreach (byte b in bytes)
            {
                this.Process(b);
            }

            return this;
        }
    }
}
