﻿//-----------------------------------------------------------------------------
// <copyright file="DDS.cs" company="eQ-3 Entwicklung GmbH">
//  Copyright (c) 2013 eQ-3 Entwicklung GmbH
// </copyright>
// <summary>Offers communication methods for DDS.</summary>
//-----------------------------------------------------------------------------
namespace DDSCommunication
{
    using System;
    using System.IO.Ports;

    /// <summary>
    /// Offers communication methods for DDS.
    /// </summary>
    public abstract class DDS : IDisposable
    {
        /// <summary>
        /// Maximal frequency of a DDS: 30MHz + two more numbers for the decimals.
        /// </summary>
        public const long MaxFrequency = 30L * 1000L * 1000L * 100L;

        /// <summary>
        /// Maximal wobbling frequency: 20 Hz + one more number for 0.1 Hz-resolution.
        /// </summary>
        public const long MaxWobblingFrequency = 200;

        /// <summary>
        /// Data bits for communication with DDS30 / DDS130.
        /// </summary>
        public const ushort Databits = 8;

        /// <summary>
        /// Parity for communication with DDS30 / DDS130.
        /// </summary>
        public const ushort Parity = 0;

        /// <summary>
        /// Stop bit for communication with DDS30 / DDS130.
        /// </summary>
        public const StopBits StopBit = StopBits.One;

        /// <summary>
        /// Timeout for read/write operations.
        /// </summary>
        public const int ReadWriteTimeout = 250;

        /// <summary>
        /// Amount of bytes in preferences.
        /// </summary>
        public const byte AmountOfPreferencesBytes = 52;

        /// <summary>
        /// The current mode of the DDS.
        /// </summary>
        private DDSPreferences.Mode currentMode = (DDSPreferences.Mode)(-1);

        /// <summary>
        /// Enum for determining the device type (not every DDS supports all used commands of this class).
        /// </summary>
        public enum DDSDevice
        {
            /// <summary>
            /// The device is a DDS 30.
            /// </summary>
            DDS30,

            /// <summary>
            /// The device is a DDS 130.
            /// </summary>
            DDS130
        }

        /// <summary>
        /// Enum of all used commands.
        /// </summary>
        public enum Opcodes
        {
            /// <summary>
            /// The command 'e' sets the current mode.
            /// </summary>
            SetMode = (byte)'e',

            /// <summary>
            /// The command 'F' gets the current frequency.
            /// </summary>
            ReadFrequency = (byte)'F',

            /// <summary>
            /// The command 'f' sets a specific frequency.
            /// </summary>
            SetFrequency = (byte)'f',

            /// <summary>
            /// The command 'P' gets the current preferences.
            /// </summary>
            ReadPreferences = (byte)'P',

            /// <summary>
            /// The command 'S' gets the current signal form.
            /// </summary>
            ReadSignalForm = (byte)'S',

            /// <summary>
            /// The command 's' sets a specific signal form.
            /// </summary>
            SetSignalForm = (byte)'s',

            /// <summary>
            /// The command 'x' activates or disables the USB-connection.
            /// </summary>
            SetUsbConnection = (byte)'x'
        }

        /// <summary>
        /// Gets or sets connected  device type (DDS30 or DDS130).
        /// </summary>
        /// <value>Connected  device type (DDS30 or DDS130).</value>
        protected DDS.DDSDevice Type { get; set; }

        /// <summary>
        /// Sends a command to the DDS and receives an answer.
        /// </summary>
        /// <param name="opcode">Opcode of the command.</param>
        /// <param name="parameter">Parameter of the command.</param>
        /// <returns>Received answer.</returns>
        public abstract byte[] SendCommand(byte opcode, byte[] parameter);

        /// <summary>
        /// Releases used resources.
        /// </summary>
        public abstract void Dispose();

        /// <summary>
        /// Sends a command to the DDS and receives an answer.
        /// </summary>
        /// <param name="opcode">Opcode of the command.</param>
        /// <returns>Received answer.</returns>
        public byte[] SendCommand(byte opcode)
        {
            return this.SendCommand(opcode, new byte[0]);
        }

        /// <summary>
        /// Sets state of the USB-connection to the DDS.
        /// </summary>
        /// <param name="activate">Indicates whether a USB-connection should activated or disabled.</param>
        /// <returns>Result of the action.</returns>
        public bool SetDeviceConnectionState(bool activate)
        {
            const byte ActivateUsbConnection = 0x01;
            const byte DisableUsbConnection = 0x00;

            byte[] result = this.SendCommand(
                DDSUtils.GetOpcodeAsByte(Opcodes.SetUsbConnection),
                new byte[] { activate ? ActivateUsbConnection : DisableUsbConnection });

            return DDSUtils.CheckResultForSetting(result, 1, true);
        }

        /// <summary>
        /// Gets the current frequency of the DDS.
        /// </summary>
        /// <returns>Current frequency of the DDS device.</returns>
        public double ReadFrequency()
        {
            const byte AmountOfFrequencyBytes = 4;

            byte[] result = this.SendCommand(
                DDSUtils.GetOpcodeAsByte(Opcodes.ReadFrequency),
                new byte[] { });

            if (!DDSUtils.CheckResultForSetting(result, AmountOfFrequencyBytes, false))
            {
                throw new InvalidOperationException("Die Frequenz konnte nicht ausgelesen werden.");
            }

            return DDSUtils.ConvertFrequency(result);
        }

        /// <summary>
        /// Sets a frequency of the DDS.
        /// </summary>
        /// <param name="wishedFrequency">Wished frequency as double.</param>
        /// <returns>Result of the action.</returns>
        public bool SetFrequency(double wishedFrequency)
        {
            long frequency = (long)(wishedFrequency * 100);
            if (frequency > MaxFrequency || frequency <= 0 || frequency % 25 != 0)
            {
                throw new ArgumentException(string.Format("Ungültige Eingabe ({0} Hz) - Wertebereich: 0,25 Hz - 30 MHz (0,25 Hz-Schritte)", wishedFrequency));
            }

            byte[] parameter = DDSUtils.ConvertFrequency(frequency);
            byte[] result = this.SendCommand(
                DDSUtils.GetOpcodeAsByte(Opcodes.SetFrequency),
                parameter);

            return DDSUtils.CheckResultForSetting(result, 1, true);
        }

        /// <summary>
        /// Gets the current signal form of the DDS.
        /// </summary>
        /// <returns>Current signal form of the DDS as enum value.</returns>
        public DDSPreferences.SignalForm ReadSignalForm()
        {
            const byte AmountOfSignalFormBytes = 1;

            byte[] result = this.SendCommand(
                DDSUtils.GetOpcodeAsByte(Opcodes.ReadSignalForm),
                new byte[] { });

            if (!DDSUtils.CheckResultForSetting(result, AmountOfSignalFormBytes, false))
            {
                throw new InvalidOperationException("Die Signalform konnte nicht ausgelesen werden.");
            }

            return (DDSPreferences.SignalForm)result[0];
        }

        /// <summary>
        /// Sets a signal of the DDS.
        /// </summary>
        /// <param name="wishedSignalForm">Wished signal form.</param>
        /// <returns>Result of the action.</returns>
        public bool SetSignalForm(DDSPreferences.SignalForm wishedSignalForm)
        {
            byte[] result = this.SendCommand(
                DDSUtils.GetOpcodeAsByte(Opcodes.SetSignalForm),
                new byte[] { (byte)wishedSignalForm });

            return DDSUtils.CheckResultForSetting(result, 1, true);
        }

        /// <summary>
        /// Gets the mode of the DDS.
        /// </summary>
        /// <returns>Read current mode.</returns>
        public DDSPreferences.Mode ReadMode()
        {
            this.currentMode = this.ReadPreferences().DeviceMode;
            return this.currentMode;
        }

        /// <summary>
        /// Reads the preferences of the DDS.
        /// </summary>
        /// <returns>Preferences of the DDS.</returns>
        public DDSPreferences ReadPreferences()
        {
            byte[] rawPreferences;
            switch (this.Type)
            {
                case DDSDevice.DDS30:
                    rawPreferences = this.SendCommand((byte)DDS.Opcodes.ReadPreferences, new byte[] { 0x00 });
                    break;
                case DDSDevice.DDS130:
                    rawPreferences = this.SendCommand((byte)DDS.Opcodes.ReadPreferences, new byte[] { });
                    break;
                default:
                    throw new InvalidOperationException();
            }

            var ddsPreferencesParser = new DDSPreferencesParser(this.Type, rawPreferences);
            return ddsPreferencesParser.Parse();
        }
    }
}