﻿//-----------------------------------------------------------------------------
// <copyright file="DDS130.cs" company="eQ-3 Entwicklung GmbH">
//  Copyright (c) 2013 eQ-3 Entwicklung GmbH
// </copyright>
// <summary>Manages USB connection and offers communication methods for DDS130.</summary>
//-----------------------------------------------------------------------------
namespace DDSCommunication
{
    using System.IO;
    using DDSCommunication.USB;

    /// <summary>
    /// Manages USB connection and offers communication methods for DDS130.
    /// </summary>
    public class DDS130 : DDS
    {
        /// <summary>
        /// Vendor ID of eQ-3.
        /// </summary>
        public const ushort VID = (ushort)0x18EF;

        /// <summary>
        /// DDS130 Product ID.
        /// </summary>
        public const ushort PID = (ushort)0xE010;

        /// <summary>
        /// Baud rate for communication with DDS130.
        /// </summary>
        public const int Baudrate = 76800;

        /// <summary>
        /// Wrapper for USB communication.
        /// </summary>
        private readonly SiUSBXpDevice usbDevice;

        /// <summary>
        /// Initializes a new instance of the DDS130 class.
        /// </summary>
        public DDS130()
        {
            this.usbDevice = new SiUSBXpDevice(
                DDS130.VID,
                DDS130.PID,
                DDS.Databits,
                DDS.Parity,
                (ushort)DDS.StopBit,
                DDS130.Baudrate);

            this.Type = DDSDevice.DDS130;

            if (!this.usbDevice.IsConnected)
            {
                throw new IOException("Es konnte keine Verbindung zum DDS hergestellt werden.");
            }
        }

        /// <summary>
        /// Sends a command to the DDS and receives an answer.
        /// </summary>
        /// <param name="opcode">Opcode of the command.</param>
        /// <param name="parameter">Parameter of the command.</param>
        /// <returns>Received answer.</returns>
        public override byte[] SendCommand(byte opcode, byte[] parameter)
        {
            byte[] commandBytes = DDSUtils.GetCommandBytes(opcode, parameter);

            byte[] rawAnswer = this.usbDevice.SendToDevice(commandBytes, DDS.ReadWriteTimeout);
            return DDSUtils.GetPayload(rawAnswer);
        }

        /// <summary>
        /// Releases used resources.
        /// </summary>
        public override void Dispose()
        {
            if (this.usbDevice != null && this.usbDevice.IsConnected)
            {
                this.usbDevice.Disconnect();
            }
        }
    }
}