﻿//-----------------------------------------------------------------------------
// <copyright file="DDS30.cs" company="eQ-3 Entwicklung GmbH">
//  Copyright (c) 2013 eQ-3 Entwicklung GmbH
// </copyright>
// <summary>Manages USB connection and offers communication methods for DDS30.</summary>
//-----------------------------------------------------------------------------
namespace DDSCommunication
{
    using System;
    using System.Collections.Generic;
    using System.IO.Ports;
    using System.Threading;

    /// <summary>
    /// Manages USB connection and offers communication methods for DDS30.
    /// </summary>
    public class DDS30 : DDS
    {
        /// <summary>
        /// Baud rate for communication with DDS30.
        /// </summary>
        public const int Baudrate = 115200;

        /// <summary>
        /// Wrapper for serial port communication.
        /// </summary>
        private SerialPort serialPortDevice;

        /// <summary>
        /// Initializes a new instance of the DDS30 class.
        /// </summary>
        /// <param name="comPort">ComPort of DDS30.</param>
        public DDS30(string comPort)
        {
            this.serialPortDevice = new SerialPort()
            {
                PortName = comPort,
                BaudRate = DDS30.Baudrate,
                Parity = DDS.Parity,
                DataBits = DDS.Databits,
                StopBits = DDS.StopBit,
                ReadTimeout = DDS.ReadWriteTimeout,
                WriteTimeout = DDS.ReadWriteTimeout
            };

            this.Type = DDSDevice.DDS30;

            this.serialPortDevice.Open();
        }

        /// <summary>
        /// Sends a command to the DDS and receives an answer.
        /// </summary>
        /// <param name="opcode">Opcode of the command.</param>
        /// <param name="parameter">Parameter of the command.</param>
        /// <returns>Received answer.</returns>
        public override byte[] SendCommand(byte opcode, byte[] parameter)
        {
            const int BytesToRead = 100;

            byte[] commandBytes = DDSUtils.GetCommandBytes(opcode, parameter);

            this.serialPortDevice.Write(commandBytes, 0, commandBytes.Length);
            this.serialPortDevice.BaseStream.Flush();
            Thread.Sleep(250);

            byte[] rawAnswer = new byte[BytesToRead];
            this.serialPortDevice.Read(rawAnswer, 0, BytesToRead);
            return DDSUtils.GetPayload(rawAnswer);
        }

        /// <summary>
        /// Releases used resources.
        /// </summary>
        public override void Dispose()
        {
            if (this.serialPortDevice != null && this.serialPortDevice.IsOpen)
            {
                this.serialPortDevice.Close();
            }
        }
    }
}
