﻿//-----------------------------------------------------------------------------
// <copyright file="DDSPreferencesParser.cs" company="eQ-3 Entwicklung GmbH">
//  Copyright (c) 2013 eQ-3 Entwicklung GmbH
// </copyright>
// <summary>Parses the read raw preferences from a DDS.</summary>
//-----------------------------------------------------------------------------
namespace DDSCommunication
{
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Text;

    /// <summary>
    /// Parses the read raw preferences from a DDS.
    /// </summary>
    internal class DDSPreferencesParser
    {
        /// <summary>
        /// Index of the version in the preferences.
        /// </summary>
        public const int VersionIndex = 0;

        /// <summary>
        /// Index of the current frequency in the preferences.
        /// </summary>
        public const int CurrentFrequencyIndex = 2;

        /// <summary>
        /// Index of the minimal frequency in the preferences.
        /// </summary>
        public const int MinimalFrequencyIndex = 6;

        /// <summary>
        /// Index of the maximal frequency in the preferences.
        /// </summary>
        public const int MaximalFrequencyIndex = 10;

        /// <summary>
        /// Index of the frequency step size in the preferences.
        /// </summary>
        public const int FrequencyStepSizeIndex = 14;

        /// <summary>
        /// Index of the amplitude in the preferences.
        /// </summary>
        public const int AmplitudeIndex = 18;

        /// <summary>
        /// Index of the wobble start frequency in the preferences.
        /// </summary>
        public const int WobbleStartFrequencyIndex = 20;

        /// <summary>
        /// Index of the wobble stop frequency in the preferences.
        /// </summary>
        public const int WobbleStopFrequencyIndex = 24;

        /// <summary>
        /// Index of the wobble frequency in the preferences.
        /// </summary>
        public const int WobbleFrequencyIndex = 28;

        /// <summary>
        /// Index of the signal form in the preferences.
        /// </summary>
        public const int SignalFormIndex = 30;

        /// <summary>
        /// Index of the mode in the preferences.
        /// </summary>
        public const byte ModeIndex = 45;

        /// <summary>
        /// Raw preferences as source for the parsing.
        /// </summary>
        public readonly byte[] RawPreferences;

        /// <summary>
        /// Amount of version bytes in the preferences.
        /// </summary>
        private const int AmountOfVersionBytes = 2;

        /// <summary>
        /// Amount of frequency bytes in the preferences.
        /// </summary>
        private const int AmountOfFrequencyBytes = 4;

        /// <summary>
        /// Amount of amplitude bytes in the preferences.
        /// </summary>
        private const int AmountOfAmplitudeBytes = 2;

        /// <summary>
        /// Amount of wobble frequency in the preferences.
        /// </summary>
        private const int AmountOfWobbleFrequencyBytes = 2;

        /// <summary>
        /// Contains only valid data with Preferences from DDS130.
        /// </summary>
        private byte[] versionBytes;

        /// <summary>
        /// Initializes a new instance of the DDSPreferencesParser class.
        /// </summary>
        /// <param name="deviceType">Type of the device from which the preferences are read from.</param>
        /// <param name="rawPreferences">Preferences read from the DDS.</param>
        public DDSPreferencesParser(DDS.DDSDevice deviceType, byte[] rawPreferences)
        {
            var adjustedRawPreferences = this.AdjustRawPreferences(deviceType, rawPreferences);
            this.RawPreferences = adjustedRawPreferences;
        }

        /// <summary>
        /// Parses preferences from raw data.
        /// </summary>
        /// <returns>Parsed preferences.</returns>
        public DDSPreferences Parse()
        {
            return new DDSPreferences()
            {
                Version = this.ParseVersion(),
                Frequency = this.ParseCurrentFrequency(),
                MinimalFrequency = this.ParseMinimalFrequency(),
                MaximalFrequency = this.ParseMaximalFrequency(),
                FrequencyStepSize = this.ParseFrequencyStepSize(),
                Amplitude = this.ParseAmplitude(),
                WobbleStartFrequency = this.ParseWobbleStartFrequency(),
                WobbleStopFrequency = this.ParseWobbleStopFrequency(),
                WobbleFrequency = this.ParseWobbleFrequency(),
                DeviceMode = this.ParseMode(),
                DeviceSignalForm = this.ParseSignalForm()
            };
        }

        /// <summary>
        /// Adjusts raw preferences: Reads data which is not send by
        /// every DDS (f.e. version info) and does possibly padding.
        /// </summary>
        /// <param name="deviceType">Type of the device (DDS30/DDS130).</param>
        /// <param name="rawPreferences">Read preferences from the device.</param>
        /// <returns>Adjusted raw preferences.</returns>
        private byte[] AdjustRawPreferences(DDS.DDSDevice deviceType, byte[] rawPreferences)
        {
            this.versionBytes = new byte[2];

            if (deviceType == DDS.DDSDevice.DDS30)
            {
                // remove first two bytes for version for compatibility with DDS30
                var adjustedRawPreferences = new byte[rawPreferences.Length + 2];
                Array.Copy(rawPreferences, 0, adjustedRawPreferences, 2, rawPreferences.Length);

                return adjustedRawPreferences;
            }
            else
            {
                // copy version info
                Array.Copy(rawPreferences, 0, this.versionBytes, 0, this.versionBytes.Length);

                // no adjustments necessary
                return rawPreferences;
            }
        }

        /// <summary>
        /// Parses the version from the raw preferences.
        /// </summary>
        /// <returns>Parsed version.</returns>
        private string ParseVersion()
        {
            const string NoVersionAvailable = "-";

            if (this.versionBytes == null ||
                this.versionBytes.Length != DDSPreferencesParser.AmountOfVersionBytes ||
                (this.versionBytes[0] == 0x00 && this.versionBytes[1] == 0x00))
            {
                return NoVersionAvailable;
            }

            int version = (this.versionBytes[0] << 8) + this.versionBytes[1];
            return string.Format("{0}.{1}", version / 100, version % 100);
        }

        /// <summary>
        /// Parses the current frequency from the raw preferences.
        /// </summary>
        /// <returns>Parsed current frequency.</returns>
        private double ParseCurrentFrequency()
        {
            return this.ConvertFrequencyFromIndex(DDSPreferencesParser.CurrentFrequencyIndex);
        }

        /// <summary>
        /// Parses the minimal frequency from the raw preferences.
        /// </summary>
        /// <returns>Parsed minimal frequency.</returns>
        private double ParseMinimalFrequency()
        {
            return this.ConvertFrequencyFromIndex(DDSPreferencesParser.MinimalFrequencyIndex);
        }

        /// <summary>
        /// Parses the maximal frequency from the raw preferences.
        /// </summary>
        /// <returns>Parsed maximal frequency.</returns>
        private double ParseMaximalFrequency()
        {
            return this.ConvertFrequencyFromIndex(DDSPreferencesParser.MaximalFrequencyIndex);
        }

        /// <summary>
        /// Parses the frequency step size from the raw preferences.
        /// </summary>
        /// <returns>Parsed frequency step size.</returns>
        private double ParseFrequencyStepSize()
        {
            return this.ConvertFrequencyFromIndex(DDSPreferencesParser.FrequencyStepSizeIndex);
        }

        /// <summary>
        /// Parses the amplitude from the raw preferences.
        /// </summary>
        /// <returns>Parsed amplitude.</returns>
        private double ParseAmplitude()
        {
            byte[] rawAmplitude = new byte[DDSPreferencesParser.AmountOfAmplitudeBytes];
            Array.Copy(this.RawPreferences, DDSPreferencesParser.AmplitudeIndex, rawAmplitude, 0, DDSPreferencesParser.AmountOfAmplitudeBytes);

            return DDSUtils.ConvertAmplitude(rawAmplitude);
        }

        /// <summary>
        /// Parses the wobble start frequency from the raw preferences.
        /// </summary>
        /// <returns>Parsed wobble start frequency.</returns>
        private double ParseWobbleStartFrequency()
        {
            return this.ConvertFrequencyFromIndex(DDSPreferencesParser.WobbleStartFrequencyIndex);
        }

        /// <summary>
        /// Parses the wobble stop frequency from the raw preferences.
        /// </summary>
        /// <returns>Parsed stop frequency.</returns>
        private double ParseWobbleStopFrequency()
        {
            return this.ConvertFrequencyFromIndex(DDSPreferencesParser.WobbleStopFrequencyIndex);
        }

        /// <summary>
        /// Converts a raw frequency. It is read from the raw preferences and the passed index.
        /// </summary>
        /// <param name="index">Start index of the frequency in the raw preferences.</param>
        /// <returns>Converted frequency.</returns>
        private double ConvertFrequencyFromIndex(int index)
        {
            byte[] rawFrequency = new byte[DDSPreferencesParser.AmountOfFrequencyBytes];
            Array.Copy(this.RawPreferences, index, rawFrequency, 0, DDSPreferencesParser.AmountOfFrequencyBytes);
            return DDSUtils.ConvertFrequency(rawFrequency);
        }

        /// <summary>
        /// Parses the wobble frequency from the raw preferences.
        /// </summary>
        /// <returns>Parsed wobble frequency.</returns>
        private double ParseWobbleFrequency()
        {
            byte[] rawWobbleFrequency = new byte[DDSPreferencesParser.AmountOfWobbleFrequencyBytes];
            Array.Copy(this.RawPreferences, DDSPreferencesParser.WobbleFrequencyIndex, rawWobbleFrequency, 0, DDSPreferencesParser.AmountOfWobbleFrequencyBytes);
            return DDSUtils.ConvertWobbleFrequency(rawWobbleFrequency);
        }

        /// <summary>
        /// Parses the signal form from the raw preferences.
        /// </summary>
        /// <returns>Parsed signal form.</returns>
        private DDSPreferences.SignalForm ParseSignalForm()
        {
            return (DDSPreferences.SignalForm)this.RawPreferences[DDSPreferencesParser.SignalFormIndex];
        }

        /// <summary>
        /// Parses the mode (standard, wobbling or modulation) from the raw preferences.
        /// </summary>
        /// <returns>Parsed mode.</returns>
        private DDSPreferences.Mode ParseMode()
        {
            DDSPreferences.Mode mode;
            switch (this.RawPreferences[DDSPreferencesParser.ModeIndex])
            {
                case 0x01:
                    mode = DDSPreferences.Mode.Standard;
                    break;
                case 0x03:
                    mode = DDSPreferences.Mode.Wobbling;
                    break;
                case 0x05:
                    mode = DDSPreferences.Mode.Modulation;
                    break;
                default:
                    throw new ArgumentException();
            }

            return mode;
        }
    }
}
