﻿//-----------------------------------------------------------------------------
// <copyright file="DDSUtils.cs" company="eQ-3 Entwicklung GmbH">
//  Copyright (c) 2013 eQ-3 Entwicklung GmbH
// </copyright>
// <summary>Utility methods for DDS30/DDS130.</summary>
//-----------------------------------------------------------------------------
namespace DDSCommunication
{
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Text;
    using DDSCommunication.Utils;

    /// <summary>
    /// Utility methods for DDS30/DDS130.
    /// </summary>
    public static class DDSUtils
    {
        /// <summary>
        /// ASCII-control-character for a start sign.
        /// </summary>
        public const byte STX = 0x02;

        /// <summary>
        /// ASCII-control-character for an acknowledge.
        /// </summary>
        public const byte ACK = 0x06;

        /// <summary>
        /// ASCII-control-character for communication.
        /// </summary>
        public const byte ENQ = 0x10;

        /// <summary>
        /// Returns a string representation of a signal form.
        /// </summary>
        /// <param name="signalForm">Signal form to convert.</param>
        /// <returns>String representation of a signal form.</returns>
        public static string SignalFormToString(DDSPreferences.SignalForm signalForm)
        {
            switch (signalForm)
            {
                case DDSPreferences.SignalForm.Sinus:
                    return "Sinus";
                case DDSPreferences.SignalForm.Triangle:
                    return "Dreieck";
                case DDSPreferences.SignalForm.RectangleFullFrequency:
                    return "Rechteck (volle Frequenz)";
                case DDSPreferences.SignalForm.RectangleHalfFrequency:
                    return "Rechteck (halbe Frequenz)";
                default:
                    throw new ArgumentException();
            }
        }

        /// <summary>
        /// Returns a string representation of the device mode.
        /// </summary>
        /// <param name="mode">Device mode.</param>
        /// <returns>String representation of the device mode.</returns>
        public static string DeviceModeToString(DDSPreferences.Mode mode)
        {
            switch (mode)
            {
                case DDSPreferences.Mode.Standard:
                    return "Standard";
                case DDSPreferences.Mode.Wobbling:
                    return "Wobbeln";
                case DDSPreferences.Mode.Modulation:
                    return "Modulation";
                default:
                    throw new ArgumentException();
            }
        }

        /// <summary>
        /// Converts a opcode as int (enum value) to a byte by casting.
        /// </summary>
        /// <param name="opcode">Opcode as enum value.</param>
        /// <returns>Opcode as byte.</returns>
        internal static byte GetOpcodeAsByte(DDS.Opcodes opcode)
        {
            return (byte)opcode;
        }

        /// <summary>
        /// Converts a byte array to frequency as double.
        /// </summary>
        /// <param name="data">Frequency as byte array.</param>
        /// <returns>Frequency as double.</returns>
        internal static double ConvertFrequency(byte[] data)
        {
            long frequence = 0;
            for (int i = 0; i < data.Length; i++)
            {
                frequence = (frequence << 8) + data[i];
            }

            return (double)frequence / 100;
        }

        /// <summary>
        /// Converts a frequency as double to a byte array.
        /// </summary>
        /// <param name="frequency">Frequency as double.</param>
        /// <returns>Frequency as byte array.</returns>
        internal static byte[] ConvertFrequency(double frequency)
        {
            byte[] data = new byte[4];
            data[0] = (byte)(frequency / Math.Pow(256, 3));
            data[1] = (byte)(frequency / Math.Pow(256, 2));
            data[2] = (byte)(frequency / Math.Pow(256, 1));
            data[3] = (byte)(frequency % 256);

            return data;
        }

        /// <summary>
        /// Converts a read raw amplitude to a number.
        /// </summary>
        /// <param name="rawAmplitude">Read raw amplitude.</param>
        /// <returns>Converted amplitude.</returns>
        internal static double ConvertAmplitude(byte[] rawAmplitude)
        {
            int frequency = (rawAmplitude[0] << 8) + rawAmplitude[1];
            return frequency;
        }

        /// <summary>
        /// Converts a read raw wobble frequency to a number.
        /// </summary>
        /// <param name="rawWobbleFrequency">Read raw wobble frequency.</param>
        /// <returns>Converted wobble frequency.</returns>
        internal static double ConvertWobbleFrequency(byte[] rawWobbleFrequency)
        {
            int wobbleFrequency = (rawWobbleFrequency[0] << 8) + rawWobbleFrequency[1];
            return (double)wobbleFrequency / 10;
        }

        /// <summary>
        /// Checks a read result of a send command.
        /// </summary>
        /// <param name="result">Read result of a send command.</param>
        /// <param name="supposedLength">Supposed length of the read result.</param>
        /// <param name="checkForAck">Indicates whether it should be checked for a acknowledge or not.</param>
        /// <returns>Result of the check.</returns>
        internal static bool CheckResultForSetting(byte[] result, int supposedLength, bool checkForAck)
        {
            if (result != null && result.Length == supposedLength)
            {
                if (checkForAck)
                {
                    if (result[0] == DDSUtils.ACK)
                    {
                        return true;
                    }
                    else
                    {
                        return false;
                    }
                }
                else
                {
                    return true;
                }
            }
            else
            {
                return false;
            }
        }

        /// <summary>
        /// Gets the byte of a command based on an opcode and parameter.
        /// </summary>
        /// <param name="opcode">Opcode of the command.</param>
        /// <param name="parameter">Parameter of the command.</param>
        /// <returns>Command based on an opcode and parameter.</returns>
        internal static byte[] GetCommandBytes(byte opcode, byte[] parameter)
        {
            List<byte> command = new List<byte>();

            // STX
            command.Add(DDSUtils.STX);

            // package number (always NULL)
            command.Add(0x00);

            // data length (Big-Endian)
            int dataLength = 1 + parameter.Length;
            command.AddRange(DDSUtils.EnqSpecialBytes(new byte[] { (byte)(dataLength % 256), (byte)(dataLength / 256) }));

            // data
            command.AddRange(DDSUtils.EnqSpecialBytes(new byte[] { opcode }));
            command.AddRange(DDSUtils.EnqSpecialBytes(parameter));

            // CRC16
            List<byte> commandListForCrc = new List<byte>(command);
            commandListForCrc.AddRange(new byte[] { 0x00, 0x00 });
            CRC16 crc = new CRC16();
            crc.Process(commandListForCrc.ToArray());

            command.Add((byte)(crc.Checksum % 256));
            command.Add((byte)(crc.Checksum / 256));

            return command.ToArray();
        }

        /// <summary>
        /// Gets the payload of a command answer.
        /// </summary>
        /// <param name="commandAnswer">Answer of a command.</param>
        /// <returns>Payload of a command answer.</returns>
        internal static byte[] GetPayload(byte[] commandAnswer)
        {
            byte[] answer = DDSUtils.TrimAnswer(commandAnswer);

            ushort readCrc = (ushort)((answer[answer.Length - 1] * 256) + answer[answer.Length - 2]);
            List<byte> crcData = new List<byte>();
            for (int i = 0; i < answer.Length; i++)
            {
                if (i < answer.Length - 2)
                {
                    crcData.Add(answer[i]);
                }
                else
                {
                    crcData.Add(0x00);
                }
            }

            CRC16 calculatedCrc = new CRC16();
            calculatedCrc.Process(crcData.ToArray());
            if (readCrc != calculatedCrc.Checksum)
            {
                return null;
            }

            answer = DDSUtils.DeEnqSpecialBytes(answer);
            return DDSUtils.GetDataFromAnswer(answer);
        }

        /// <summary>
        /// Converts a read raw frequency deviation to a number.
        /// </summary>
        /// <param name="fskFrequencyDeviation">Read raw frequency deviation.</param>
        /// <returns>Converted frequency deviation.</returns>
        private static double ConvertRawFrequencyDeviation(byte[] fskFrequencyDeviation)
        {
            long frequencyDeviation = 0;

            for (int i = 0; i < fskFrequencyDeviation.Length; i++)
            {
                frequencyDeviation += fskFrequencyDeviation[i] << (8 * i);
            }

            return (double)frequencyDeviation / 100;
        }

        /// <summary>
        /// Escape special bytes like STX and ENQ.
        /// </summary>
        /// <param name="buffer">Data from application.</param>
        /// <returns>Encoded data.</returns>
        private static byte[] EnqSpecialBytes(byte[] buffer)
        {
            List<byte> temp = new List<byte>();

            for (int i = 0; i < buffer.Length; i++)
            {
                if (buffer[i] == STX)
                {
                    temp.Add(DDSUtils.ENQ);
                    temp.Add(STX + 0x80);
                }
                else if (buffer[i] == DDSUtils.ENQ)
                {
                    temp.Add(DDSUtils.ENQ);
                    temp.Add(DDSUtils.ENQ + 0x80);
                }
                else
                {
                    temp.Add(buffer[i]);
                }
            }

            return temp.ToArray();
        }

        /// <summary>
        /// Remove escape of special bytes like STX and ENQ.
        /// </summary>
        /// <param name="buffer">Encoded data from device.</param>
        /// <returns>Data for application.</returns>
        private static byte[] DeEnqSpecialBytes(byte[] buffer)
        {
            List<byte> temp = new List<byte>();

            for (int i = 0; i < buffer.Length; i++)
            {
                if (buffer[i] == STX)
                {
                }
                else if (buffer[i] == DDSUtils.ENQ)
                {
                    temp.Add((byte)(buffer[i + 1] - 0x80));
                    i++;
                }
                else
                {
                    temp.Add(buffer[i]);
                }
            }

            return temp.ToArray();
        }

        /// <summary>
        /// Removes unnecessary 0x00 at the end of the received answer.
        /// </summary>
        /// <param name="answer">Received answer.</param>
        /// <returns>Trimmed answer.</returns>
        private static byte[] TrimAnswer(byte[] answer)
        {
            List<byte> answerList = new List<byte>(answer);

            for (int i = answerList.Count - 1; i >= 0; i--)
            {
                if (answer[i] == 0x00)
                {
                    answerList.RemoveAt(i);
                }
                else
                {
                    break;
                }
            }

            return answerList.ToArray();
        }

        /// <summary>
        /// Gets the data of the answer, filters out unnecessary bytes.
        /// </summary>
        /// <param name="answer">Received answer of a command.</param>
        /// <returns>Filtered answer.</returns>
        private static byte[] GetDataFromAnswer(byte[] answer)
        {
            List<byte> answerAsList = new List<byte>(answer);

            const int OFFSET_BEFORE_DATA = 4;
            const int OFFSET_AFTER_DATA = 2;

            List<byte> data = new List<byte>();

            for (int i = OFFSET_BEFORE_DATA; i < answerAsList.Count - OFFSET_AFTER_DATA; i++)
            {
                data.Add(answer[i]);
            }

            return data.ToArray();
        }
    }
}